@@ -7,9 +7,20 @@
 *** from the company.
 ***
 **/
+
+/* This file is intended to "cover" native compilers that don't provide it.
+   The tests below must be adjusted if/when a native compiler provides its
+   own. Only supports up to C++17. */
+
 #ifndef __USE_INTEL_CATOMICS
   #if _MSC_VER > 0 && _MSC_VER < 1931
     #define __USE_INTEL_CATOMICS 1
+  #elif _MSC_VER > 0 && !defined(__cpp_consteval) && !defined(_ATOMIC_)
+    /* Enable for MS C,C++11,17 */
+    /* Disable if user inadvertently included <atomic> also */
+    #define __USE_INTEL_CATOMICS 1
+  #elif _MSC_VER > 0 && defined(_M_CEE_PURE)
+    #define __USE_INTEL_CATOMICS 1
   #elif defined(__INTEL_LLVM_COMPILER)
     #define __USE_INTEL_CATOMICS 0
   #elif __GNUC__ > 0 && (__GNUC__ < 4 || __GNUC__ == 4 && __GNUC_MINOR__ < 9) && \
