@@ -94,7 +94,7 @@
             #define frexpf        __MS_frexpf
             #define hypotf        __MS_hypotf
 
-            #if !defined(__INTEL_LLVM_COMPILER) || (defined(__AVX512FP16__) && defined(__INTEL_LLVM_COMPILER))
+            #if !defined(__INTEL_LLVM_COMPILER) || ((defined(__AVX512F__) || (__AVX256P_SUPPORTED__)) && defined(__INTEL_LLVM_COMPILER))
                 #define ldexpf    __MS_ldexpf
             #endif
 
