@@ -21,8 +21,8 @@
 
 #   define KMP_VERSION_MAJOR    5
 #   define KMP_VERSION_MINOR    0
-#   define KMP_VERSION_BUILD    20230118
-#   define KMP_BUILD_DATE       "2023-01-18 22:26:52 UTC"
+#   define KMP_VERSION_BUILD    20230428
+#   define KMP_BUILD_DATE       "2023-05-03 17-06-27 UTC"
 
 #   ifdef __cplusplus
     extern "C" {
@@ -178,7 +178,12 @@
         omp_ipr_device_eu_simd_width = 2,
         omp_ipr_device_num_eus_per_subslice = 3,
         omp_ipr_device_num_subslices_per_slice = 4,
-        omp_ipr_device_num_slices = 5
+        omp_ipr_device_num_slices = 5,
+        omp_ipr_device_local_mem_size = 6,
+        omp_ipr_device_global_mem_size = 7,
+        omp_ipr_device_global_mem_cache_size = 8,
+        omp_ipr_device_max_clock_frequency = 9,
+        omp_ipr_is_imm_cmd_list = 10
     } omp_interop_property_t;
 
     #define omp_interop_none 0
@@ -370,10 +375,12 @@
     extern __KMP_IMP omp_allocator_handle_t const omp_cgroup_mem_alloc;
     extern __KMP_IMP omp_allocator_handle_t const omp_pteam_mem_alloc;
     extern __KMP_IMP omp_allocator_handle_t const omp_thread_mem_alloc;
-    /* Extension for target memory support */
     extern __KMP_IMP omp_allocator_handle_t const omp_target_host_mem_alloc;
     extern __KMP_IMP omp_allocator_handle_t const omp_target_shared_mem_alloc;
     extern __KMP_IMP omp_allocator_handle_t const omp_target_device_mem_alloc;
+    extern __KMP_IMP omp_allocator_handle_t const llvm_omp_target_host_mem_alloc;
+    extern __KMP_IMP omp_allocator_handle_t const llvm_omp_target_shared_mem_alloc;
+    extern __KMP_IMP omp_allocator_handle_t const llvm_omp_target_device_mem_alloc;
 
     typedef omp_uintptr_t omp_memspace_handle_t;
     extern __KMP_IMP omp_memspace_handle_t const omp_default_mem_space;
@@ -381,10 +388,12 @@
     extern __KMP_IMP omp_memspace_handle_t const omp_const_mem_space;
     extern __KMP_IMP omp_memspace_handle_t const omp_high_bw_mem_space;
     extern __KMP_IMP omp_memspace_handle_t const omp_low_lat_mem_space;
-    /* Extension for target memory support */
     extern __KMP_IMP omp_memspace_handle_t const omp_target_host_mem_space;
     extern __KMP_IMP omp_memspace_handle_t const omp_target_shared_mem_space;
     extern __KMP_IMP omp_memspace_handle_t const omp_target_device_mem_space;
+    extern __KMP_IMP omp_memspace_handle_t const llvm_omp_target_host_mem_space;
+    extern __KMP_IMP omp_memspace_handle_t const llvm_omp_target_shared_mem_space;
+    extern __KMP_IMP omp_memspace_handle_t const llvm_omp_target_device_mem_space;
 #   else
 #       if __cplusplus >= 201103
     typedef enum omp_allocator_handle_t : omp_uintptr_t
@@ -401,10 +410,12 @@
       omp_cgroup_mem_alloc = 6,
       omp_pteam_mem_alloc = 7,
       omp_thread_mem_alloc = 8,
-      /* Extension for target memory support */
-      omp_target_host_mem_alloc = 100,
-      omp_target_shared_mem_alloc = 101,
-      omp_target_device_mem_alloc = 102,
+      llvm_omp_target_host_mem_alloc = 100,
+      llvm_omp_target_shared_mem_alloc = 101,
+      llvm_omp_target_device_mem_alloc = 102,
+      omp_target_host_mem_alloc = llvm_omp_target_host_mem_alloc,
+      omp_target_shared_mem_alloc = llvm_omp_target_shared_mem_alloc,
+      omp_target_device_mem_alloc = llvm_omp_target_device_mem_alloc,
       KMP_ALLOCATOR_MAX_HANDLE = UINTPTR_MAX
     } omp_allocator_handle_t;
 #       if __cplusplus >= 201103
@@ -418,10 +429,12 @@
       omp_const_mem_space = 2,
       omp_high_bw_mem_space = 3,
       omp_low_lat_mem_space = 4,
-      /* Extension for target memory support */
-      omp_target_host_mem_space = 100,
-      omp_target_shared_mem_space = 101,
-      omp_target_device_mem_space = 102,
+      llvm_omp_target_host_mem_space = 100,
+      llvm_omp_target_shared_mem_space = 101,
+      llvm_omp_target_device_mem_space = 102,
+      omp_target_host_mem_space = llvm_omp_target_host_mem_space,
+      omp_target_shared_mem_space = llvm_omp_target_shared_mem_space,
+      omp_target_device_mem_space = llvm_omp_target_device_mem_space,
       KMP_MEMSPACE_MAX_HANDLE = UINTPTR_MAX
     } omp_memspace_handle_t;
 #   endif
@@ -501,6 +514,9 @@
     /* OpenMP 5.2 */
     extern int __KAI_KMPC_CONVENTION omp_in_explicit_task(void);
 
+    /* LLVM Extensions */
+    extern void *llvm_omp_target_dynamic_shared_alloc();
+
     /* Extension for named barrier access in device code */
     extern void __KAI_KMPC_CONVENTION ompx_nbarrier_init(uint32_t nbarrier_count);
     extern void __KAI_KMPC_CONVENTION ompx_nbarrier_wait(uint32_t nbarrier_id);
@@ -564,6 +580,9 @@
     extern int __KAI_KMPC_CONVENTION ompx_target_register_host_pointer(void *ptr, size_t size, int device_num);
     extern void __KAI_KMPC_CONVENTION ompx_target_unregister_host_pointer(void *ptr, int device_num);
 
+    /* Extened API routine that returns device ID from a memory location */
+    extern int __KAI_KMPC_CONVENTION ompx_get_device_from_ptr(const void *);
+
 #   undef __KAI_KMPC_CONVENTION
 #   undef __KMP_IMP
 
