@@ -14,7 +14,7 @@
 
 #if defined(__SYCL_DEVICE_ONLY__) && defined(SYCL_FALLBACK_ASSERT)
 // Reads Flag of AssertHappened on device
-SYCL_EXTERNAL __attribute__((weak)) extern "C" void
+__DPCPP_SYCL_EXTERNAL __attribute__((weak)) extern "C" void
 __devicelib_assert_read(void *);
 #endif
 
