@@ -1,16 +1,3 @@
-//
-// Modifications, Copyright (C) 2021 Intel Corporation
-//
-// This software and the related documents are Intel copyrighted materials, and
-// your use of them is governed by the express license under which they were
-// provided to you ("License"). Unless the License provides otherwise, you may not
-// use, modify, copy, publish, distribute, disclose or transmit this software or
-// the related documents without Intel's prior written permission.
-//
-// This software and the related documents are provided as is, with no express
-// or implied warranties, other than those that are expressly stated in the
-// License.
-//
 //==-------------- CG.hpp - SYCL standard header file ----------------------==//
 //
 // Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
@@ -85,6 +72,9 @@ public:
     Copy2DUSM = 16,
     Fill2DUSM = 17,
     Memset2DUSM = 18,
+    CopyToDeviceGlobal = 19,
+    CopyFromDeviceGlobal = 20,
+    ReadWriteHostPipe = 21,
   };
 
   CG(CGTYPE Type, std::vector<std::vector<char>> ArgsStorage,
@@ -111,6 +101,10 @@ public:
 
   CGTYPE getType() { return MType; }
 
+  std::vector<std::vector<char>> &getArgsStorage() { return MArgsStorage; }
+
+  std::vector<detail::AccessorImplPtr> &getAccStorage() { return MAccStorage; }
+
   virtual ~CG() = default;
 
 private:
@@ -151,6 +145,7 @@ public:
   detail::OSModuleHandle MOSModuleHandle;
   std::vector<std::shared_ptr<detail::stream_impl>> MStreams;
   std::vector<std::shared_ptr<const void>> MAuxiliaryResources;
+  RT::PiKernelCacheConfig MKernelCacheConfig;
 
   CGExecKernel(NDRDescT NDRDesc, std::unique_ptr<HostKernelBase> HKernel,
                std::shared_ptr<detail::kernel_impl> SyclKernel,
@@ -164,7 +159,8 @@ public:
                detail::OSModuleHandle OSModuleHandle,
                std::vector<std::shared_ptr<detail::stream_impl>> Streams,
                std::vector<std::shared_ptr<const void>> AuxiliaryResources,
-               CGTYPE Type, detail::code_location loc = {})
+               CGTYPE Type, RT::PiKernelCacheConfig KernelCacheConfig,
+               detail::code_location loc = {})
       : CG(Type, std::move(ArgsStorage), std::move(AccStorage),
            std::move(SharedPtrStorage), std::move(Requirements),
            std::move(Events), std::move(loc)),
@@ -173,7 +169,8 @@ public:
         MKernelBundle(std::move(KernelBundle)), MArgs(std::move(Args)),
         MKernelName(std::move(KernelName)), MOSModuleHandle(OSModuleHandle),
         MStreams(std::move(Streams)),
-        MAuxiliaryResources(std::move(AuxiliaryResources)) {
+        MAuxiliaryResources(std::move(AuxiliaryResources)),
+        MKernelCacheConfig(std::move(KernelCacheConfig)) {
     assert((getType() == RunOnHostIntel || getType() == Kernel) &&
            "Wrong type of exec kernel CG.");
   }
@@ -499,6 +496,104 @@ public:
   char getValue() const { return MValue; }
 };
 
+/// "ReadWriteHostPipe" command group class.
+class CGReadWriteHostPipe : public CG {
+  std::string PipeName;
+  bool Blocking;
+  void *HostPtr;
+  size_t TypeSize;
+  bool IsReadOp;
+
+public:
+  CGReadWriteHostPipe(const std::string &Name, bool Block, void *Ptr,
+                      size_t Size, bool Read,
+                      std::vector<std::vector<char>> ArgsStorage,
+                      std::vector<detail::AccessorImplPtr> AccStorage,
+                      std::vector<std::shared_ptr<const void>> SharedPtrStorage,
+                      std::vector<AccessorImplHost *> Requirements,
+                      std::vector<detail::EventImplPtr> Events,
+                      detail::code_location loc = {})
+      : CG(ReadWriteHostPipe, std::move(ArgsStorage), std::move(AccStorage),
+           std::move(SharedPtrStorage), std::move(Requirements),
+           std::move(Events), std::move(loc)),
+        PipeName(Name), Blocking(Block), HostPtr(Ptr), TypeSize(Size),
+        IsReadOp(Read) {}
+
+  std::string getPipeName() { return PipeName; }
+  void *getHostPtr() { return HostPtr; }
+  size_t getTypeSize() { return TypeSize; }
+  bool isBlocking() { return Blocking; }
+  bool isReadHostPipe() { return IsReadOp; }
+};
+
+/// "Copy to device_global" command group class.
+class CGCopyToDeviceGlobal : public CG {
+  void *MSrc;
+  void *MDeviceGlobalPtr;
+  bool MIsDeviceImageScoped;
+  size_t MNumBytes;
+  size_t MOffset;
+  detail::OSModuleHandle MOSModuleHandle;
+
+public:
+  CGCopyToDeviceGlobal(
+      void *Src, void *DeviceGlobalPtr, bool IsDeviceImageScoped,
+      size_t NumBytes, size_t Offset,
+      std::vector<std::vector<char>> ArgsStorage,
+      std::vector<detail::AccessorImplPtr> AccStorage,
+      std::vector<std::shared_ptr<const void>> SharedPtrStorage,
+      std::vector<AccessorImplHost *> Requirements,
+      std::vector<detail::EventImplPtr> Events,
+      detail::OSModuleHandle OSModuleHandle, detail::code_location loc = {})
+      : CG(CopyToDeviceGlobal, std::move(ArgsStorage), std::move(AccStorage),
+           std::move(SharedPtrStorage), std::move(Requirements),
+           std::move(Events), std::move(loc)),
+        MSrc(Src), MDeviceGlobalPtr(DeviceGlobalPtr),
+        MIsDeviceImageScoped(IsDeviceImageScoped), MNumBytes(NumBytes),
+        MOffset(Offset), MOSModuleHandle(OSModuleHandle) {}
+
+  void *getSrc() { return MSrc; }
+  void *getDeviceGlobalPtr() { return MDeviceGlobalPtr; }
+  bool isDeviceImageScoped() { return MIsDeviceImageScoped; }
+  size_t getNumBytes() { return MNumBytes; }
+  size_t getOffset() { return MOffset; }
+  detail::OSModuleHandle getOSModuleHandle() { return MOSModuleHandle; }
+};
+
+/// "Copy to device_global" command group class.
+class CGCopyFromDeviceGlobal : public CG {
+  void *MDeviceGlobalPtr;
+  void *MDest;
+  bool MIsDeviceImageScoped;
+  size_t MNumBytes;
+  size_t MOffset;
+  detail::OSModuleHandle MOSModuleHandle;
+
+public:
+  CGCopyFromDeviceGlobal(
+      void *DeviceGlobalPtr, void *Dest, bool IsDeviceImageScoped,
+      size_t NumBytes, size_t Offset,
+      std::vector<std::vector<char>> ArgsStorage,
+      std::vector<detail::AccessorImplPtr> AccStorage,
+      std::vector<std::shared_ptr<const void>> SharedPtrStorage,
+      std::vector<AccessorImplHost *> Requirements,
+      std::vector<detail::EventImplPtr> Events,
+      detail::OSModuleHandle OSModuleHandle, detail::code_location loc = {})
+      : CG(CopyFromDeviceGlobal, std::move(ArgsStorage), std::move(AccStorage),
+           std::move(SharedPtrStorage), std::move(Requirements),
+           std::move(Events), std::move(loc)),
+        MDeviceGlobalPtr(DeviceGlobalPtr), MDest(Dest),
+        MIsDeviceImageScoped(IsDeviceImageScoped), MNumBytes(NumBytes),
+        MOffset(Offset), MOSModuleHandle(OSModuleHandle) {}
+
+  void *getDeviceGlobalPtr() { return MDeviceGlobalPtr; }
+  void *getDest() { return MDest; }
+  bool isDeviceImageScoped() { return MIsDeviceImageScoped; }
+  size_t getNumBytes() { return MNumBytes; }
+  size_t getOffset() { return MOffset; }
+  detail::OSModuleHandle getOSModuleHandle() { return MOSModuleHandle; }
+};
+
 } // namespace detail
 } // __SYCL_INLINE_VER_NAMESPACE(_V1)
 } // namespace sycl
