@@ -35,7 +35,8 @@ public:
     (void)_Success;
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -52,7 +53,8 @@ public:
     (void)_Data;
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -69,7 +71,8 @@ public:
 #else
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead..");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -84,7 +87,8 @@ public:
     (void)_Data;
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -137,7 +141,8 @@ public:
     (void)_Success;
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -154,7 +159,8 @@ public:
 #else
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -187,7 +193,8 @@ public:
     (void)_Success;
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
@@ -203,7 +210,8 @@ public:
     (void)_Data;
     throw sycl::exception(
         sycl::make_error_code(sycl::errc::feature_not_supported),
-        "Pipes are not supported on a host device.");
+        "Device-side API are not supported on a host device. Please use "
+        "host-side API instead.");
 #endif // __SYCL_DEVICE_ONLY__
   }
 
