@@ -15,6 +15,7 @@
 #include <sycl/exception.hpp>
 #include <sycl/ext/oneapi/device_global/properties.hpp>
 #include <sycl/ext/oneapi/properties/properties.hpp>
+#include <sycl/pointers.hpp>
 
 #ifdef __SYCL_DEVICE_ONLY__
 #define __SYCL_HOST_NOT_SUPPORTED(Op)
@@ -42,9 +43,31 @@ struct HasArrowOperator<
 template <typename T, typename PropertyListT, typename = void>
 class device_global_base {
 protected:
-  T *usmptr;
-  T *get_ptr() noexcept { return usmptr; }
-  const T *get_ptr() const noexcept { return usmptr; }
+  using pointer_t = typename decorated_global_ptr<T>::pointer;
+
+  // The pointer member is mutable to avoid the compiler optimizing it out when
+  // accessing const-qualified device_global variables.
+  mutable pointer_t usmptr{};
+
+  pointer_t get_ptr() noexcept { return usmptr; }
+  const pointer_t get_ptr() const noexcept { return usmptr; }
+
+public:
+  template <access::decorated IsDecorated>
+  multi_ptr<T, access::address_space::global_space, IsDecorated>
+  get_multi_ptr() noexcept {
+    __SYCL_HOST_NOT_SUPPORTED("get_multi_ptr()")
+    return multi_ptr<T, access::address_space::global_space, IsDecorated>{
+        get_ptr()};
+  }
+
+  template <access::decorated IsDecorated>
+  multi_ptr<const T, access::address_space::global_space, IsDecorated>
+  get_multi_ptr() const noexcept {
+    __SYCL_HOST_NOT_SUPPORTED("get_multi_ptr()")
+    return multi_ptr<const T, access::address_space::global_space, IsDecorated>{
+        get_ptr()};
+  }
 };
 
 // Specialization of device_global base class for when device_image_scope is in
@@ -58,6 +81,23 @@ protected:
   T val{};
   T *get_ptr() noexcept { return &val; }
   const T *get_ptr() const noexcept { return &val; }
+
+public:
+  template <access::decorated IsDecorated>
+  multi_ptr<T, access::address_space::global_space, IsDecorated>
+  get_multi_ptr() noexcept {
+    __SYCL_HOST_NOT_SUPPORTED("get_multi_ptr()")
+    return address_space_cast<access::address_space::global_space, IsDecorated,
+                              T>(this->get_ptr());
+  }
+
+  template <access::decorated IsDecorated>
+  multi_ptr<const T, access::address_space::global_space, IsDecorated>
+  get_multi_ptr() const noexcept {
+    __SYCL_HOST_NOT_SUPPORTED("get_multi_ptr()")
+    return address_space_cast<access::address_space::global_space, IsDecorated,
+                              const T>(this->get_ptr());
+  }
 };
 } // namespace detail
 
@@ -99,13 +139,6 @@ public:
   static_assert(is_property_list<property_list_t>::value,
                 "Property list is invalid.");
 
-  // TODO: Remove when support has been added for device_global without the
-  // device_image_scope property.
-  static_assert(
-      property_list_t::template has_property<device_image_scope_key>(),
-      "device_global without the device_image_scope property is currently "
-      "unavailable.");
-
   device_global() = default;
 
   device_global(const device_global &) = delete;
@@ -113,22 +146,6 @@ public:
   device_global &operator=(const device_global &) = delete;
   device_global &operator=(const device_global &&) = delete;
 
-  template <access::decorated IsDecorated>
-  multi_ptr<T, access::address_space::global_space, IsDecorated>
-  get_multi_ptr() noexcept {
-    __SYCL_HOST_NOT_SUPPORTED("get_multi_ptr()")
-    return address_space_cast<access::address_space::global_space, IsDecorated>(
-        this->get_ptr());
-  }
-
-  template <access::decorated IsDecorated>
-  multi_ptr<const T, access::address_space::global_space, IsDecorated>
-  get_multi_ptr() const noexcept {
-    __SYCL_HOST_NOT_SUPPORTED("get_multi_ptr()")
-    return address_space_cast<access::address_space::global_space, IsDecorated,
-                              const T>(this->get_ptr());
-  }
-
   T &get() noexcept {
     __SYCL_HOST_NOT_SUPPORTED("get()")
     return *this->get_ptr();
