@@ -221,6 +221,23 @@ template <typename LHSPropertiesT, typename RHSPropertiesT>
 using merged_properties_t =
     typename merged_properties<LHSPropertiesT, RHSPropertiesT>::type;
 
+template <typename Properties, typename PropertyKey, typename Cond = void>
+struct ValueOrDefault {
+  template <typename ValT> static constexpr ValT get(ValT Default) {
+    return Default;
+  }
+};
+
+template <typename Properties, typename PropertyKey>
+struct ValueOrDefault<
+    Properties, PropertyKey,
+    std::enable_if_t<is_property_list_v<Properties> &&
+                     Properties::template has_property<PropertyKey>()>> {
+  template <typename ValT> static constexpr ValT get(ValT) {
+    return Properties::template get_property<PropertyKey>().value;
+  }
+};
+
 } // namespace detail
 } // namespace ext::oneapi::experimental
 
@@ -232,12 +249,5 @@ struct is_device_copyable<
     std::enable_if_t<!std::is_trivially_copyable<
         ext::oneapi::experimental::properties<PropertiesT>>::value>>
     : is_device_copyable<PropertiesT> {};
-template <typename PropertiesT>
-struct is_device_copyable<
-    const ext::oneapi::experimental::properties<PropertiesT>,
-    std::enable_if_t<!std::is_trivially_copyable<
-        const ext::oneapi::experimental::properties<PropertiesT>>::value>>
-    : is_device_copyable<PropertiesT> {};
-
 } // __SYCL_INLINE_VER_NAMESPACE(_V1)
 } // namespace sycl
