@@ -140,12 +140,8 @@ public:
 #ifdef __SYCL_DEVICE_ONLY__
     return __spirv::initLocalInvocationId<Dimensions, id<Dimensions>>();
 #else
-    throw runtime_error("get_local_id() is not implemented on host device",
+    throw runtime_error("get_local_id() is not implemented on host",
                         PI_ERROR_INVALID_DEVICE);
-    // Implementing get_local_id() on host device requires ABI breaking change.
-    // It requires extending class group with local item which represents
-    // local_id. Currently this local id is only used in nd_item and group
-    // cannot access it.
 #endif
   }
 
@@ -445,15 +441,15 @@ private:
   typename detail::enable_if_t<(dims == 2), size_t>
   get_local_linear_id_impl() const {
     id<Dimensions> localId = get_local_id();
-    return localId[0] * groupRange[1] + localId[1];
+    return localId[0] * localRange[1] + localId[1];
   }
 
   template <int dims = Dimensions>
   typename detail::enable_if_t<(dims == 3), size_t>
   get_local_linear_id_impl() const {
     id<Dimensions> localId = get_local_id();
-    return (localId[0] * groupRange[1] * groupRange[2]) +
-           (localId[1] * groupRange[2]) + localId[2];
+    return (localId[0] * localRange[1] * localRange[2]) +
+           (localId[1] * localRange[2]) + localId[2];
   }
 
   template <int dims = Dimensions>
@@ -558,7 +554,7 @@ group<Dims> this_group() {
 #else
   throw sycl::exception(
       sycl::make_error_code(sycl::errc::feature_not_supported),
-      "Free function calls are not supported on host device");
+      "Free function calls are not supported on host");
 #endif
 }
 
@@ -570,7 +566,7 @@ template <int Dims> group<Dims> this_group() {
 #else
   throw sycl::exception(
       sycl::make_error_code(sycl::errc::feature_not_supported),
-      "Free function calls are not supported on host device");
+      "Free function calls are not supported on host");
 #endif
 }
 } // namespace ext::oneapi::experimental
