@@ -9,10 +9,11 @@
 #pragma once
 
 #include <sycl/aspects.hpp>
+#include <sycl/bit_cast.hpp>
 #include <sycl/detail/defines.hpp>
 #include <sycl/detail/export.hpp>
 #include <sycl/detail/iostream_proxy.hpp>
-#include <sycl/detail/type_traits.hpp>
+#include <sycl/detail/vector_traits.hpp>
 
 #include <functional>
 #include <limits>
@@ -33,6 +34,10 @@
 
 namespace sycl {
 __SYCL_INLINE_VER_NAMESPACE(_V1) {
+namespace detail::half_impl {
+class half;
+}
+using half = detail::half_impl::half;
 
 namespace ext::intel::esimd::detail {
 class WrapperElementTypeProxy;
@@ -248,10 +253,15 @@ using BIsRepresentationT = half;
 // hood. As a result half values will be converted to the integer and passed
 // as a kernel argument which is expected to be floating point number.
 template <int NumElements> struct half_vec {
-  alignas(detail::vector_alignment<StorageT, NumElements>::value)
-      StorageT s[NumElements];
+  alignas(
+      vector_alignment<StorageT, NumElements>::value) StorageT s[NumElements];
 
   __SYCL_CONSTEXPR_HALF half_vec() : s{0.0f} { initialize_data(); }
+  template <typename... Ts,
+            typename = std::enable_if_t<(sizeof...(Ts) == NumElements) &&
+                                        (std::is_same_v<half, Ts> && ...)>>
+  __SYCL_CONSTEXPR_HALF half_vec(const Ts &...hs) : s{hs...} {}
+
   constexpr void initialize_data() {
     for (size_t i = 0; i < NumElements; ++i) {
       s[i] = StorageT(0.0f);
