@@ -148,7 +148,7 @@ item<Dims> this_item() {
 #else
   throw sycl::exception(
       sycl::make_error_code(sycl::errc::feature_not_supported),
-      "Free function calls are not supported on host device");
+      "Free function calls are not supported on host");
 #endif
 }
 
@@ -159,7 +159,7 @@ template <int Dims> item<Dims> this_item() {
 #else
   throw sycl::exception(
       sycl::make_error_code(sycl::errc::feature_not_supported),
-      "Free function calls are not supported on host device");
+      "Free function calls are not supported on host");
 #endif
 }
 } // namespace ext::oneapi::experimental
