@@ -14,17 +14,11 @@
 
 namespace sycl {
 __SYCL_INLINE_VER_NAMESPACE(_V1) {
-namespace property {
-namespace reduction {
+namespace property::reduction {
 class initialize_to_identity
     : public detail::DataLessProperty<detail::InitializeToIdentity> {};
-} // namespace reduction
-} // namespace property
+} // namespace property::reduction
 
 // Reduction property trait specializations
-template <>
-struct is_property<property::reduction::initialize_to_identity>
-    : std::true_type {};
-
 } // __SYCL_INLINE_VER_NAMESPACE(_V1)
 } // namespace sycl
