@@ -228,14 +228,80 @@ inline unsigned append(char *Dst, const char *Src) {
   return Len;
 }
 
+static inline unsigned F2I32(float Val) {
+  union {
+    float FVal;
+    unsigned I32Val;
+  } Internal;
+  Internal.FVal = Val;
+  return Internal.I32Val;
+}
+
+static inline unsigned long long D2I64(double Val) {
+  union {
+    double DVal;
+    unsigned long long I64Val;
+  } Internal;
+  Internal.DVal = Val;
+  return Internal.I64Val;
+}
+
 template <typename T>
 inline typename detail::enable_if_t<
+    std::is_same<T, float>::value || std::is_same<T, double>::value, bool>
+isFastMathInf(T Val) {
+  if constexpr (sizeof(Val) == 4) {
+    return (F2I32(Val) & 0x7fffffff) == 0x7f800000;
+  } else if constexpr (sizeof(Val) == 8) {
+    return (D2I64(Val) & -1ULL >> 1) == 0x7ffULL << 52;
+  }
+
+  return false;
+}
+
+template <typename T>
+inline typename detail::enable_if_t<
+    std::is_same<T, float>::value || std::is_same<T, double>::value, bool>
+isFastMathNan(T Val) {
+  if constexpr (sizeof(Val) == 4) {
+    return (F2I32(Val) & 0x7fffffff) > 0x7f800000;
+  } else if constexpr (sizeof(Val) == 8) {
+    return (D2I64(Val) & -1ULL >> 1) > 0x7ffULL << 52;
+  }
+
+  return false;
+}
+
+template <typename T>
+inline typename detail::enable_if_t<
+    std::is_same<T, float>::value || std::is_same<T, double>::value, bool>
+isFastMathSignBit(T Val) {
+  if constexpr (sizeof(Val) == 4) {
+    return F2I32(Val) >> 31;
+  } else if constexpr (sizeof(Val) == 8) {
+    return D2I64(Val) >> 63;
+  }
+
+  return false;
+}
+
+template <typename T>
+typename detail::enable_if_t<
     std::is_same<T, float>::value || std::is_same<T, double>::value, unsigned>
 checkForInfNan(char *Buf, T Val) {
+#ifdef __FAST_MATH__
+  if (isFastMathNan(Val))
+#else
   if (isnan(Val))
+#endif
     return append(Buf, "nan");
+#ifdef __FAST_MATH__
+  if (isFastMathInf(Val)) {
+    if (isFastMathSignBit(Val))
+#else
   if (isinf(Val)) {
     if (signbit(Val))
+#endif
       return append(Buf, "-inf");
     return append(Buf, "inf");
   }
@@ -743,6 +809,17 @@ inline __width_manipulator__ setw(int Width) {
 /// \ingroup sycl_api
 class __SYCL_EXPORT __SYCL_SPECIAL_CLASS __SYCL_TYPE(stream) stream
     : public detail::OwnerLessBase<stream> {
+private:
+#ifndef __SYCL_DEVICE_ONLY__
+  // Constructor for recreating a stream.
+  stream(std::shared_ptr<detail::stream_impl> Impl,
+         detail::GlobalBufAccessorT GlobalBuf,
+         detail::GlobalOffsetAccessorT GlobalOffset,
+         detail::GlobalBufAccessorT GlobalFlushBuf)
+      : impl{Impl}, GlobalBuf{GlobalBuf}, GlobalOffset{GlobalOffset},
+        GlobalFlushBuf{GlobalFlushBuf} {}
+#endif
+
 public:
 #ifdef __SYCL_DEVICE_ONLY__
   // Default constructor for objects later initialized with __init member.
@@ -757,9 +834,36 @@ public:
   stream(size_t BufferSize, size_t MaxStatementSize, handler &CGH,
          const property_list &PropList);
 
+#ifdef __SYCL_DEVICE_ONLY__
+  // We need the definitions of these functions in the header for device,
+  // otherwise they are not visible. Also, we cannot use `impl` because it's not
+  // there on the device, so we rely on GlobalBuf/GlobalFlushBuf.
+  size_t size() const noexcept { return GlobalBuf.size(); }
+
+  size_t get_work_item_buffer_size() const {
+    return GlobalFlushBuf.size() - detail::FLUSH_BUF_OFFSET_SIZE;
+  }
+
+  __SYCL2020_DEPRECATED(
+      "get_size() is deprecated since SYCL 2020. Please use size() instead.")
+  size_t get_size() const { return size(); }
+
+  __SYCL2020_DEPRECATED("get_max_statement_size() is deprecated since SYCL "
+                        "2020. Please use get_work_item_buffer_size() instead.")
+  size_t get_max_statement_size() const { return get_work_item_buffer_size(); }
+#else
+  size_t size() const noexcept;
+
+  size_t get_work_item_buffer_size() const;
+
+  __SYCL2020_DEPRECATED(
+      "get_size() is deprecated since SYCL 2020. Please use size() instead.")
   size_t get_size() const;
 
+  __SYCL2020_DEPRECATED("get_max_statement_size() is deprecated since SYCL "
+                        "2020. Please use get_work_item_buffer_size() instead.")
   size_t get_max_statement_size() const;
+#endif
 
   size_t get_precision() const { return Precision; }
 
@@ -784,6 +888,10 @@ private:
   friend decltype(Obj::impl) detail::getSyclObjImpl(const Obj &SyclObject);
 #endif
 
+  // NOTE: Some members are required for reconstructing the stream, but are not
+  // part of the implementation class. If more members are added, they should
+  // also be added to the weak_object specialization for streams.
+
   // Accessor to the global stream buffer. Global buffer contains all output
   // from the kernel.
   mutable detail::GlobalBufAccessorT GlobalBuf;
@@ -915,6 +1023,8 @@ private:
 
   friend class handler;
 
+  template <typename SYCLObjT> friend class ext::oneapi::weak_object;
+
   friend const stream &operator<<(const stream &, const char);
   friend const stream &operator<<(const stream &, const char *);
   template <typename ValueType>
